@@ -1,20 +1,21 @@
 from dataclasses import dataclass
 import io
-from lmdbm import Lmdb
 import numpy as np
 import numpy.typing as npt
 from pathlib import Path
 from typing import Generator, Iterable, Tuple, Union
 
 from .db import DbFactory, DbWrapper
+from .dna import AbstractSequenceWrapper
+from .types import int_t
 from .utils import open_file
 
-def phred_encode(probabilities: npt.ArrayLike, encoding: int = 33) -> str:
+def phred_encode(probabilities: npt.ArrayLike, encoding: int_t = 33) -> str:
     scores = (-10 * np.log10(np.array(probabilities))).astype(int)
     return ''.join((chr(score + encoding)) for score in scores)
 
 
-def phred_decode(qualities: str, encoding: int = 33) -> npt.NDArray[np.float64]:
+def phred_decode(qualities: str, encoding: int_t = 33) -> npt.NDArray[np.float64]:
     scores = np.array([(ord(token) - encoding) for token in qualities])
     return 10**(scores / -10)
 
@@ -38,14 +39,14 @@ class FastqHeader:
     )
 
     instrument: str
-    run_number: int
+    run_number: int_t
     flowcell_id: str
-    lane: int
-    tile: int
+    lane: int_t
+    tile: int_t
     pos: Tuple[int, int]
-    read_type: int
+    read_type: int_t
     is_filtered: bool
-    control_number: int
+    control_number: int_t
     sequence_index: str
 
     @classmethod
@@ -96,15 +97,14 @@ class FastqHeader:
 
 
 @dataclass(frozen=True, order=True)
-class FastqEntry:
+class FastqEntry(AbstractSequenceWrapper):
     """
     A class representation of a FASTQ entry containing the sequnce identifier, sequence, and quality
     scores.
     """
-    __slots__ = ("header_str", "sequence", "quality_scores")
+    __slots__ = ("header_str", "quality_scores")
 
     header_str: str
-    sequence: str
     quality_scores: str
 
     @classmethod
@@ -116,6 +116,11 @@ class FastqEntry:
         header, sequence, _, quality_scores= entry.rstrip().split('\n')
         return cls(header, sequence, quality_scores)
 
+    def __init__(self, header: str, sequence: str, quality_scores: str):
+        object.__setattr__(self, "header_str", header)
+        object.__setattr__(self, "sequence", sequence)
+        object.__setattr__(self, "quality_scores", quality_scores)
+
     def serialize(self) -> bytes:
         return '\x00'.join((self.header_str, self.sequence, self.quality_scores)).encode()
 
@@ -131,7 +136,7 @@ class FastqDbFactory(DbFactory):
     """
     A factory for creating LMDB-backed databases of FASTA entries.
     """
-    def __init__(self, path: Union[str, Path], chunk_size: int = 10000):
+    def __init__(self, path: Union[str, Path], chunk_size: int_t = 10000):
         super().__init__(path, chunk_size)
         self.num_entries = np.int32(0)
 
@@ -159,14 +164,14 @@ class FastqDb(DbWrapper):
     def __len__(self):
         return self.length
 
-    def __contains__(self, sequence_index: int) -> bool:
+    def __contains__(self, sequence_index: int_t) -> bool:
         return str(sequence_index) in self.db
 
     def __iter__(self):
         for i in range(len(self)):
             yield self[i]
 
-    def __getitem__(self, sequence_index: int) -> FastqEntry:
+    def __getitem__(self, sequence_index: int_t) -> FastqEntry:
         return FastqEntry.deserialize(self.db[str(sequence_index)])
 
 
@@ -183,7 +188,6 @@ def entries(
         yield from read(sequences)
     else:
         yield from sequences
-    raise TypeError(f"Unsupported type: {type(sequences)}")
 
 
 def read(buffer: io.TextIOBase) -> Generator[FastqEntry, None, None]:
